@@ -29,6 +29,12 @@
 namespace
 {
 
+rx::RobustnessVideoMemoryPurgeStatus GetRobustnessVideoMemoryPurge(const egl::AttributeMap &attribs)
+{
+    return static_cast<rx::RobustnessVideoMemoryPurgeStatus>(
+        attribs.get(GLX_GENERATE_RESET_ON_VIDEO_MEMORY_PURGE_NV, GL_FALSE));
+}
+
 bool HasParallelShaderCompileExtension(const rx::FunctionsGL *functions)
 {
     return functions->maxShaderCompilerThreadsKHR != nullptr ||
@@ -68,7 +74,8 @@ DisplayGLX::DisplayGLX(const egl::DisplayState &state)
       mVisuals(nullptr),
       mContext(nullptr),
       mSharedContext(nullptr),
-      mDummyPbuffer(0),
+      mCurrentNativeContexts(),
+      mInitPbuffer(0),
       mUsesNewXDisplay(false),
       mIsMesa(false),
       mHasMultisample(false),
@@ -76,6 +83,7 @@ DisplayGLX::DisplayGLX(const egl::DisplayState &state)
       mHasARBCreateContextProfile(false),
       mHasARBCreateContextRobustness(false),
       mHasEXTCreateContextES2Profile(false),
+      mHasNVRobustnessVideoMemoryPurge(false),
       mSwapControl(SwapControl::Absent),
       mMinSwapInterval(0),
       mMaxSwapInterval(0),
@@ -115,8 +123,9 @@ egl::Error DisplayGLX::initialize(egl::Display *display)
     mHasMultisample             = mGLX.minorVersion > 3 || mGLX.hasExtension("GLX_ARB_multisample");
     mHasARBCreateContext        = mGLX.hasExtension("GLX_ARB_create_context");
     mHasARBCreateContextProfile = mGLX.hasExtension("GLX_ARB_create_context_profile");
-    mHasARBCreateContextRobustness = mGLX.hasExtension("GLX_ARB_create_context_robustness");
-    mHasEXTCreateContextES2Profile = mGLX.hasExtension("GLX_EXT_create_context_es2_profile");
+    mHasARBCreateContextRobustness   = mGLX.hasExtension("GLX_ARB_create_context_robustness");
+    mHasEXTCreateContextES2Profile   = mGLX.hasExtension("GLX_EXT_create_context_es2_profile");
+    mHasNVRobustnessVideoMemoryPurge = mGLX.hasExtension("GLX_NV_robustness_video_memory_purge");
 
     std::string clientVendor = mGLX.getClientString(GLX_VENDOR);
     mIsMesa                  = clientVendor.find("Mesa") != std::string::npos;
@@ -158,7 +167,6 @@ egl::Error DisplayGLX::initialize(egl::Display *display)
     if (attribMap.contains(EGL_X11_VISUAL_ID_ANGLE))
     {
         mRequestedVisual = static_cast<EGLint>(attribMap.get(EGL_X11_VISUAL_ID_ANGLE, -1));
-
         // There is no direct way to get the GLXFBConfig matching an X11 visual ID
         // so we have to iterate over all the GLXFBConfigs to find the right one.
         int nConfigs;
@@ -257,7 +265,7 @@ egl::Error DisplayGLX::initialize(egl::Display *display)
     }
     ASSERT(mContext);
 
-    mCurrentContexts[std::this_thread::get_id()] = mContext;
+    mCurrentNativeContexts[std::this_thread::get_id()] = mContext;
 
     // FunctionsGL and DisplayGL need to make a few GL calls, for example to
     // query the version of the context so we need to make the context current.
@@ -269,23 +277,30 @@ egl::Error DisplayGLX::initialize(egl::Display *display)
     // as commented on https://bugs.freedesktop.org/show_bug.cgi?id=38869 so we
     // use (1, 1) instead.
 
-    int dummyPbufferAttribs[] = {
+    int initPbufferAttribs[] = {
         GLX_PBUFFER_WIDTH, 1, GLX_PBUFFER_HEIGHT, 1, None,
     };
-    mDummyPbuffer = mGLX.createPbuffer(mContextConfig, dummyPbufferAttribs);
-    if (!mDummyPbuffer)
+    mInitPbuffer = mGLX.createPbuffer(mContextConfig, initPbufferAttribs);
+    if (!mInitPbuffer)
     {
-        return egl::EglNotInitialized() << "Could not create the dummy pbuffer.";
+        return egl::EglNotInitialized() << "Could not create the initialization pbuffer.";
     }
 
-    if (!mGLX.makeCurrent(mDummyPbuffer, mContext))
+    if (!mGLX.makeCurrent(mInitPbuffer, mContext))
     {
-        return egl::EglNotInitialized() << "Could not make the dummy pbuffer current.";
+        return egl::EglNotInitialized() << "Could not make the initialization pbuffer current.";
     }
 
     std::unique_ptr<FunctionsGL> functionsGL(new FunctionsGLGLX(mGLX.getProc));
     functionsGL->initialize(eglAttributes);
-
+    if (mHasNVRobustnessVideoMemoryPurge)
+    {
+        GLenum status = functionsGL->getGraphicsResetStatus();
+        if (status != GL_NO_ERROR && status != GL_PURGED_CONTEXT_RESET_NV)
+        {
+            return egl::EglNotInitialized() << "Context lost for unknown reason.";
+        }
+    }
     // TODO(cwallez, angleproject:1303) Disable the OpenGL ES backend on Linux NVIDIA and Intel as
     // it has problems on our automated testing. An OpenGL ES backend might not trigger this test if
     // there is no Desktop OpenGL support, but that's not the case in our automated testing.
@@ -309,8 +324,7 @@ egl::Error DisplayGLX::initialize(egl::Display *display)
         {
             for (unsigned int i = 0; i < RendererGL::getMaxWorkerContexts(); ++i)
             {
-                glx::Pbuffer workerPbuffer =
-                    mGLX.createPbuffer(mContextConfig, dummyPbufferAttribs);
+                glx::Pbuffer workerPbuffer = mGLX.createPbuffer(mContextConfig, initPbufferAttribs);
                 if (!workerPbuffer)
                 {
                     return egl::EglNotInitialized() << "Could not create the worker pbuffers.";
@@ -320,7 +334,7 @@ egl::Error DisplayGLX::initialize(egl::Display *display)
         }
     }
 
-    syncXCommands();
+    syncXCommands(false);
 
     mRenderer.reset(new RendererGLX(std::move(functionsGL), eglAttributes, this));
     const gl::Version &maxVersion = mRenderer->getMaxSupportedESVersion();
@@ -342,10 +356,10 @@ void DisplayGLX::terminate()
         mVisuals = 0;
     }
 
-    if (mDummyPbuffer)
+    if (mInitPbuffer)
     {
-        mGLX.destroyPbuffer(mDummyPbuffer);
-        mDummyPbuffer = 0;
+        mGLX.destroyPbuffer(mInitPbuffer);
+        mInitPbuffer = 0;
     }
 
     for (auto &workerPbuffer : mWorkerPbufferPool)
@@ -354,7 +368,7 @@ void DisplayGLX::terminate()
     }
     mWorkerPbufferPool.clear();
 
-    mCurrentContexts.clear();
+    mCurrentNativeContexts.clear();
 
     if (mContext)
     {
@@ -378,12 +392,13 @@ void DisplayGLX::terminate()
     }
 }
 
-egl::Error DisplayGLX::makeCurrent(egl::Surface *drawSurface,
+egl::Error DisplayGLX::makeCurrent(egl::Display *display,
+                                   egl::Surface *drawSurface,
                                    egl::Surface *readSurface,
                                    gl::Context *context)
 {
     glx::Drawable newDrawable =
-        (drawSurface ? GetImplAs<SurfaceGLX>(drawSurface)->getDrawable() : mDummyPbuffer);
+        (drawSurface ? GetImplAs<SurfaceGLX>(drawSurface)->getDrawable() : mInitPbuffer);
     glx::Context newContext = mContext;
     // If the thread calling makeCurrent does not have the correct context current (either mContext
     // or 0), we need to set it current.
@@ -393,17 +408,17 @@ egl::Error DisplayGLX::makeCurrent(egl::Surface *drawSurface,
         newContext  = 0;
     }
     if (newDrawable != mCurrentDrawable ||
-        newContext != mCurrentContexts[std::this_thread::get_id()])
+        newContext != mCurrentNativeContexts[std::this_thread::get_id()])
     {
         if (mGLX.makeCurrent(newDrawable, newContext) != True)
         {
             return egl::EglContextLost() << "Failed to make the GLX context current";
         }
-        mCurrentContexts[std::this_thread::get_id()] = newContext;
-        mCurrentDrawable                             = newDrawable;
+        mCurrentNativeContexts[std::this_thread::get_id()] = newContext;
+        mCurrentDrawable                                   = newDrawable;
     }
 
-    return DisplayGL::makeCurrent(drawSurface, readSurface, context);
+    return DisplayGL::makeCurrent(display, drawSurface, readSurface, context);
 }
 
 SurfaceImpl *DisplayGLX::createWindowSurface(const egl::SurfaceState &state,
@@ -447,7 +462,7 @@ SurfaceImpl *DisplayGLX::createPixmapSurface(const egl::SurfaceState &state,
     return new PixmapSurfaceGLX(state, nativePixmap, mGLX.getDisplay(), mGLX, fbConfig);
 }
 
-egl::Error DisplayGLX::validatePixmap(egl::Config *config,
+egl::Error DisplayGLX::validatePixmap(const egl::Config *config,
                                       EGLNativePixmapType pixmap,
                                       const egl::AttributeMap &attributes) const
 {
@@ -475,13 +490,9 @@ ContextImpl *DisplayGLX::createContext(const gl::State &state,
                                        const gl::Context *shareContext,
                                        const egl::AttributeMap &attribs)
 {
-    return new ContextGL(state, errorSet, mRenderer);
-}
-
-DeviceImpl *DisplayGLX::createDevice()
-{
-    UNIMPLEMENTED();
-    return nullptr;
+    RobustnessVideoMemoryPurgeStatus robustnessVideoMemoryPurgeStatus =
+        GetRobustnessVideoMemoryPurge(attribs);
+    return new ContextGL(state, errorSet, mRenderer, robustnessVideoMemoryPurgeStatus);
 }
 
 egl::Error DisplayGLX::initializeContext(glx::FBConfig config,
@@ -776,12 +787,6 @@ bool DisplayGLX::isValidNativeWindow(EGLNativeWindowType window) const
     return status != 0;
 }
 
-std::string DisplayGLX::getVendorString() const
-{
-    // UNIMPLEMENTED();
-    return "";
-}
-
 egl::Error DisplayGLX::waitClient(const gl::Context *context)
 {
     mGLX.waitGL();
@@ -820,9 +825,9 @@ gl::Version DisplayGLX::getMaxSupportedESVersion() const
     return mRenderer->getMaxSupportedESVersion();
 }
 
-void DisplayGLX::syncXCommands() const
+void DisplayGLX::syncXCommands(bool alwaysSync) const
 {
-    if (mUsesNewXDisplay)
+    if (mUsesNewXDisplay || alwaysSync)
     {
         XSync(mGLX.getDisplay(), False);
     }
@@ -871,26 +876,37 @@ void DisplayGLX::setSwapInterval(glx::Drawable drawable, SwapControlData *data)
     }
 }
 
-bool DisplayGLX::isValidWindowVisualId(unsigned long visualId) const
+bool DisplayGLX::isWindowVisualIdSpecified() const
 {
-    return mRequestedVisual == -1 || static_cast<unsigned long>(mRequestedVisual) == visualId;
+    return mRequestedVisual != -1;
+}
+
+bool DisplayGLX::isMatchingWindowVisualId(unsigned long visualId) const
+{
+    return isWindowVisualIdSpecified() && static_cast<unsigned long>(mRequestedVisual) == visualId;
 }
 
 void DisplayGLX::generateExtensions(egl::DisplayExtensions *outExtensions) const
 {
     outExtensions->createContextRobustness = mHasARBCreateContextRobustness;
 
-    // Contexts are virtualized so textures can be shared globally
-    outExtensions->displayTextureShareGroup = true;
+    // Contexts are virtualized so textures ans semaphores can be shared globally
+    outExtensions->displayTextureShareGroup   = true;
+    outExtensions->displaySemaphoreShareGroup = true;
 
     outExtensions->surfacelessContext = true;
 
-    const bool hasSyncControlOML        = mGLX.hasExtension("GLX_OML_sync_control");
-    outExtensions->syncControlCHROMIUM  = hasSyncControlOML;
-    outExtensions->syncControlRateANGLE = hasSyncControlOML;
+    if (!mRenderer->getFeatures().disableSyncControlSupport.enabled)
+    {
+        const bool hasSyncControlOML        = mGLX.hasExtension("GLX_OML_sync_control");
+        outExtensions->syncControlCHROMIUM  = hasSyncControlOML;
+        outExtensions->syncControlRateANGLE = hasSyncControlOML;
+    }
 
     outExtensions->textureFromPixmapNOK = mGLX.hasExtension("GLX_EXT_texture_from_pixmap");
 
+    outExtensions->robustnessVideoMemoryPurgeNV = mHasNVRobustnessVideoMemoryPurge;
+
     DisplayGL::generateExtensions(outExtensions);
 }
 
@@ -926,6 +942,11 @@ egl::Error DisplayGLX::createContextAttribs(glx::FBConfig,
         mAttribs.push_back(GLX_CONTEXT_ROBUST_ACCESS_BIT_ARB);
         mAttribs.push_back(GLX_CONTEXT_RESET_NOTIFICATION_STRATEGY_ARB);
         mAttribs.push_back(GLX_LOSE_CONTEXT_ON_RESET_ARB);
+        if (mHasNVRobustnessVideoMemoryPurge)
+        {
+            mAttribs.push_back(GLX_GENERATE_RESET_ON_VIDEO_MEMORY_PURGE_NV);
+            mAttribs.push_back(GL_TRUE);
+        }
     }
 
     if (version.valid())
@@ -1053,4 +1074,14 @@ void DisplayGLX::populateFeatureList(angle::FeatureList *features)
     mRenderer->getFeatures().populateFeatureList(features);
 }
 
+RendererGL *DisplayGLX::getRenderer() const
+{
+    return mRenderer.get();
+}
+
+bool DisplayGLX::isX11() const
+{
+    return true;
+}
+
 }  // namespace rx
